@@ -13,8 +13,8 @@ RVineAIC <-function(data,RVM,par=RVM$par,par2=RVM$par2){
 	if(n != dim(RVM)) stop("Dimensions of 'data' and 'RVM' do not match.")
   if(!is(RVM,"RVineMatrix")) stop("'RVM' has to be an RVineMatrix object.")
   
-	npar = sum(RVM$family >= 1, na.rm=TRUE) + sum(RVM$family %in% c(2,7:10,17:20,27:30,37:40),na.rm=TRUE)
-  npar_pair = (RVM$family>=1)+(RVM$family%in%c(2,7:10,17:20,27:30,37:40))
+	npar = sum(RVM$family >= 1, na.rm=TRUE) + sum(RVM$family %in% c(2,7:10,17:20,27:30,37:40,104,114,124,134,204,214,224,234),na.rm=TRUE)
+  npar_pair = (RVM$family>=1)+(RVM$family%in%c(2,7:10,17:20,27:30,37:40,104,114,124,134,204,214,224,234))
 
   like = RVineLogLik(data,RVM)
 
@@ -38,8 +38,8 @@ RVineBIC <-function(data,RVM,par=RVM$par,par2=RVM$par2){
 	if(n != dim(RVM)) stop("Dimensions of 'data' and 'RVM' do not match.")
   if(is(RVM) != "RVineMatrix") stop("'RVM' has to be an RVineMatrix object.")
   
-	npar = sum(RVM$family >= 1, na.rm=TRUE) + sum(RVM$family %in% c(2,7:10,17:20,27:30,37:40),na.rm=TRUE)
-  npar_pair = (RVM$family>=1)+(RVM$family%in%c(2,7:10,17:20,27:30,37:40))
+	npar = sum(RVM$family >= 1, na.rm=TRUE) + sum(RVM$family %in% c(2,7:10,17:20,27:30,37:40,104,114,124,134,204,214,224,234),na.rm=TRUE)
+  npar_pair = (RVM$family>=1)+(RVM$family%in%c(2,7:10,17:20,27:30,37:40,104,114,124,134,204,214,224,234))
 
   like = RVineLogLik(data,RVM)
 
